window.Gleam.initSearch([{"doc":"totally","title":"totally","content":"# totally\n\n[![Package Version](https://img.shields.io/hexpm/v/totally)](https://hex.pm/packages/totally)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/totally/)\n\ntotally (TOTP+ally) is a TOTP (Time-based One-Time Password) library for Gleam (Erlang &amp; Javascript(Node) targets).\n\nImplements the [TOTP RFC6238](https://datatracker.ietf.org/doc/html/rfc6238)\n\nInspired by [NimbleTOTP](https://github.com/dashbitco/nimble_totp) and [OTPAuth](https://github.com/hectorm/otpauth)\n\n## Installation\n\n```sh\ngleam add totally\n```\n\n## Basic Usage\n\n```gleam\nimport totally\n\npub fn main() {\n  // Create user secret. This should be stored securely.\n  let secret = totally.secret()\n\n  // Generate an OTP auth URI. Display this as a QR code to the user.\n  totally.otpauth_uri(secret, issuer: &quot;totally&quot;, account: &quot;joe&quot;)\n  // =&gt; &quot;otpauth://totp/totally:joe?secret=JKVVN7MCLQ4OJFTNCZUGAESASCDAJII2&amp;issuer=totally&amp;algorithm=SHA1&amp;digits=6&amp;period=30&quot;\n\n  // Generate a TOTP if you need to send it to the user via another channel\n  totally.totp(secret)\n  // =&gt; OTP(&quot;492755&quot;)\n\n  let user_input = &quot;492755&quot;\n\n  // Verify a TOTP\n  totally.verify(secret, user_input)\n  // =&gt; true\n}\n```\n\n## Advanced Usage\n\n```gleam\nimport totally\n\npub fn main() {\n  let secret = totally.secret()\n\n  // Using the builder pattern\n  let config =\n    totally.default_config()\n    |&gt; totally.set_secret(secret)\n    |&gt; totally.set_time_now\n    |&gt; totally.set_issuer(&quot;totally&quot;)\n    |&gt; totally.set_account(&quot;joe&quot;)\n\n  // or the TOTPConfig type directly\n  let config =\n    TOTPConfig(\n      secret: secret,\n      time: 1_723_813_617,\n      algorithm: totally.SHA1,\n      digits: 6,\n      period: 30,\n      issuer: &quot;totally&quot;,\n      account: &quot;joe&quot;,\n    )\n\n  totally.otpauth_uri_from_config(config)\n\n  totally.totp_from_config(config)\n\n  let user_input = &quot;492755&quot;\n\n  totally.verify_from_config(config, user_input)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/totally&gt;.\n\n## Development\n\n```sh\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"totally","title":"OTP","content":"pub opaque type OTP\n One Time Password type\n\nSynonyms:\ntotally.OTP\ntotally OTP","url":"totally.html#OTP"},{"doc":"totally","title":"TOTPAlgorithm","content":"pub type TOTPAlgorithm {\n  Sha1\n  Sha256\n  Sha512\n}\n Algorithm used for the hash function\nSha1\n\n\nSha256\n\n\nSha512\n\n\nSynonyms:\ntotally.TOTPAlgorithm\ntotally TOTPAlgorithm","url":"totally.html#TOTPAlgorithm"},{"doc":"totally","title":"TOTPConfig","content":"pub type TOTPConfig {\n  TOTPConfig(\n    secret: Secret,\n    time: Int,\n    period: Int,\n    digits: Int,\n    algorithm: TOTPAlgorithm,\n    issuer: String,\n    account: String,\n  )\n}\n Configuration for the TOTP\nTOTPConfig(\n  secret: Secret,\n  time: Int,\n  period: Int,\n  digits: Int,\n  algorithm: TOTPAlgorithm,\n  issuer: String,\n  account: String,\n)\n\n\nSynonyms:\ntotally.TOTPConfig\ntotally TOTPConfig","url":"totally.html#TOTPConfig"},{"doc":"totally","title":"default_config","content":"pub fn default_config() -&gt; TOTPConfig\n Creates a default configuration for TOTP with the following values:\n algorithm: Sha1, period: 30, digits: 6. These are the most commonly used TOTP settings.\n Please set the secret and time with the `set_secret` and `set_time_now` or manually `set_time` functions.\nSynonyms:\ntotally.default_config\ntotally default_config","url":"totally.html#default_config"},{"doc":"totally","title":"otp_to_string","content":"pub fn otp_to_string(otp: OTP) -&gt; String\n Converts the OTP to a string.\nSynonyms:\ntotally.otp_to_string\ntotally otp_to_string","url":"totally.html#otp_to_string"},{"doc":"totally","title":"otpauth_uri","content":"pub fn otpauth_uri(\n  secret secret: BitArray,\n  issuer issuer: String,\n  account account_name: String,\n) -&gt; String\n Generates an otpauth URI for the given secret, issuer and account name.\n The otpauth URI is used to generate QR codes for TOTP.\nSynonyms:\ntotally.otpauth_uri\ntotally otpauth_uri","url":"totally.html#otpauth_uri"},{"doc":"totally","title":"otpauth_uri_from_config","content":"pub fn otpauth_uri_from_config(config: TOTPConfig) -&gt; String\n Generates an otpauth URI for the given TOTP configuration.\nSynonyms:\ntotally.otpauth_uri_from_config\ntotally otpauth_uri_from_config","url":"totally.html#otpauth_uri_from_config"},{"doc":"totally","title":"secret","content":"pub fn secret() -&gt; BitArray\n Generates a random 20 byte secret.\n 20 bytes is the recommended size according to the HOTP RFC4226 (https://tools.ietf.org/html/rfc4226#section-4).\nSynonyms:\ntotally.secret\ntotally secret","url":"totally.html#secret"},{"doc":"totally","title":"secret_with_size","content":"pub fn secret_with_size(size: Int) -&gt; BitArray\n Generates a random secret with the given size.\nSynonyms:\ntotally.secret_with_size\ntotally secret_with_size","url":"totally.html#secret_with_size"},{"doc":"totally","title":"set_account","content":"pub fn set_account(\n  config: TOTPConfig,\n  account: String,\n) -&gt; TOTPConfig\n Sets the account for the TOTP configuration.\nSynonyms:\ntotally.set_account\ntotally set_account","url":"totally.html#set_account"},{"doc":"totally","title":"set_algorithm","content":"pub fn set_algorithm(\n  config: TOTPConfig,\n  algorithm: TOTPAlgorithm,\n) -&gt; TOTPConfig\n Sets the algorithm for the TOTP configuration.\n Most commonly used is Sha1.\nSynonyms:\ntotally.set_algorithm\ntotally set_algorithm","url":"totally.html#set_algorithm"},{"doc":"totally","title":"set_digits","content":"pub fn set_digits(config: TOTPConfig, digits: Int) -&gt; TOTPConfig\n Sets the digits for the TOTP configuration.\n Most commonly used is 6 digits.\n The spec allows for 6 to 8 digits.\nSynonyms:\ntotally.set_digits\ntotally set_digits","url":"totally.html#set_digits"},{"doc":"totally","title":"set_issuer","content":"pub fn set_issuer(\n  config: TOTPConfig,\n  issuer: String,\n) -&gt; TOTPConfig\n Sets the issuer for the TOTP configuration.\n Used for the otpauth URI.\nSynonyms:\ntotally.set_issuer\ntotally set_issuer","url":"totally.html#set_issuer"},{"doc":"totally","title":"set_period","content":"pub fn set_period(config: TOTPConfig, period: Int) -&gt; TOTPConfig\n Sets the refresh period in seconds for the TOTP configuration.\n Most commonly used is 30 seconds.\nSynonyms:\ntotally.set_period\ntotally set_period","url":"totally.html#set_period"},{"doc":"totally","title":"set_secret","content":"pub fn set_secret(\n  config: TOTPConfig,\n  secret: BitArray,\n) -&gt; TOTPConfig\n Sets the secret for the TOTP configuration.\nSynonyms:\ntotally.set_secret\ntotally set_secret","url":"totally.html#set_secret"},{"doc":"totally","title":"set_time","content":"pub fn set_time(config: TOTPConfig, time: Int) -&gt; TOTPConfig\n Sets the time in unix timestamp seconds for the TOTP configuration.\nSynonyms:\ntotally.set_time\ntotally set_time","url":"totally.html#set_time"},{"doc":"totally","title":"set_time_now","content":"pub fn set_time_now(config: TOTPConfig) -&gt; TOTPConfig\n Sets the time for the TOTP configuration to the current time.\nSynonyms:\ntotally.set_time_now\ntotally set_time_now","url":"totally.html#set_time_now"},{"doc":"totally","title":"string_to_otp","content":"pub fn string_to_otp(otp: String) -&gt; Result(OTP, String)\n Converts a valid OTP string to an OTP type.\nSynonyms:\ntotally.string_to_otp\ntotally string_to_otp","url":"totally.html#string_to_otp"},{"doc":"totally","title":"totp","content":"pub fn totp(secret: BitArray) -&gt; OTP\n Generates a TOTP using the given secret and default configuration.\nSynonyms:\ntotally.totp\ntotally totp","url":"totally.html#totp"},{"doc":"totally","title":"totp_from_config","content":"pub fn totp_from_config(config: TOTPConfig) -&gt; OTP\n Generates a TOTP using the given TOTP configuration.\nSynonyms:\ntotally.totp_from_config\ntotally totp_from_config","url":"totally.html#totp_from_config"},{"doc":"totally","title":"verify","content":"pub fn verify(\n  secret secret: BitArray,\n  input totp_input: String,\n) -&gt; Bool\n Verifies the given TOTP input with the given secret.\nSynonyms:\ntotally.verify\ntotally verify","url":"totally.html#verify"},{"doc":"totally","title":"verify_from_config","content":"pub fn verify_from_config(\n  config: TOTPConfig,\n  input totp_input: String,\n) -&gt; Bool\n Verifies the given TOTP input with the given TOTP configuration.\nSynonyms:\ntotally.verify_from_config\ntotally verify_from_config","url":"totally.html#verify_from_config"},{"doc":"totally","title":"totally","content":"","url":"totally.html"}]);